from __future__ import unicode_literals
import nltk
from . import TextStatistics as stats
import math
from .Silabizator import englishSilabizer
from .styleTokenizer import styleTokenizer

def ARI(text, contract = True):
    """Calculates the Automated Readability Index of an English text.
    
    The function returns the two available ARI's metrics of the library.
    The first is the clasical ARI and the second is the optimized ARI.
    
    """

    tokens = styleTokenizer(text, lang='en', contract = contract)
    sentenceNumber = stats.sentencesCount(text,lang = 'en')
    wordsNumber = len(tokens)
    
    charNumber = 0
    for token in tokens:
        charNumber += len(token)

    avgCharPerWord = charNumber/wordsNumber
    avgWordsPerSentence = wordsNumber/sentenceNumber

    ari = math.ceil(5.84*(avgCharPerWord) + 0.37*(avgWordsPerSentence) - 26.01)
            
    return ari


def fleschRI(text, contract = True):
    """Calculates the Flesch Readability Index of an English text.
    
    This function returns the two available Flesch's metrics of the library.
    The first is the Flesch Reading Ease Index and the second is the
    Flesch-Kincaid Grade Level Index.
    
    """
    
    tokens = styleTokenizer(text, lang = 'en', contract = contract)
    sentenceNumber = stats.sentencesCount(text,lang = 'en')
    wordsNumber = len(tokens)

    syllablesNumber = 0
    s = englishSilabizer()
    for token in tokens:
        syllablesNumber += s(token)    

    avgSyllPerWord = syllablesNumber/wordsNumber
    avgWordsPerSentence = wordsNumber/sentenceNumber

    fleschKincaidGradeLevel = 0.39*(avgWordsPerSentence) + 11.8*(avgSyllPerWord) - 15.59
    
    return fleschKincaidGradeLevel

def fogCount(text, contract = True):
    """Calculates the Fog Count Index of an English text.
    
    The function returns the two available Fog Count's metrics of the library.
    The first is the clasical Fog Count and the second is the optimized Fog Count.
    
    """

    tokens = styleTokenizer(text, lang = 'en', contract = contract)
    sentenceNumber = stats.sentencesCount(text,lang = 'en')
    wordsNumber = len(tokens)
    newFogCount = 0
    easyWords = []
    hardWords = []
    s = englishSilabizer()

    for token in tokens:
        if s(token) > 2:
            hardWords.append(token)
        else:
            easyWords.append(token)

    avgFogCount = (len(easyWords) + 3 * len(hardWords))/sentenceNumber
    
    newFogCount = (avgFogCount - 3)/2
                
    return newFogCount


from __future__ import unicode_literals
import nltk
from . import TextStatistics as stats
import math
from .Silabizator import spanishSilabizer
from .styleTokenizer import styleTokenizer

def INFLESZ(text):
    """Calculates the INFLESZ Index of a Spanish text.
    
    The function returns the INFLESZ Readability Index value of a
    text written in Spanish.
    
    """

    tokens = styleTokenizer(text, lang = 'es')
    sentenceNumber = stats.sentencesCount(text,lang = 'es')
    wordsNumber = len(tokens)
    s = spanishSilabizer()
    syllablesNumber = 0

    for token in tokens:
        syllablesNumber += len(s(token))
    
    inflesz = 206.835 - ((62.3*syllablesNumber)/wordsNumber)-(wordsNumber/sentenceNumber)
    
    return inflesz

def muReadability(text):
    """Calculates the mu Readability Index of a Spanish text.
    
    The function returns the mu Readbility Index of a text written
    in Spanish.
    
    """

    tokens = styleTokenizer(text, lang = 'es', punct = True)
    charsPerWord = stats.charactersPerWord(text, lang = 'es', punct = True)
    wordsNumber = len(tokens)
    variance = 0
    
    for token in tokens:
        variance += ((len(token)-charsPerWord)**2)
        
    variance /= wordsNumber
    
    mu = ((wordsNumber/(wordsNumber - 1)) * (charsPerWord/variance)) * 100
        
    return mu   


from . import TextStatistics as stats
from . import englishRIAlgorithms as riEn
from . import spanishRIAlgorithms as riEs
from . import VocabularyRichness as vr
from . import formality as form
from . import coherenceMetrics as coh

class StyleMetrics():

	def _setReadabilityIndex(self, text, lang, contract):


		if lang == "en":
			readabilityIndexes = {'ARI': riEn.ARI(text, contract), 
							'FleschRI': riEn.fleschRI(text, contract),
							'FogCount': riEn.fogCount(text, contract)
							}
		else:
			readabilityIndexes = {'INFLESZ': riEs.INFLESZ(text),
							'Mu': riEs.muReadability(text)
							}

		return readabilityIndexes

	def _setTextStatistics(self, text, lang, lowercase, punct, contract, stopWords, blank):

		diffWords = stats.differentWords(text, lang, lowercase, contract, punct, stopWords)
		#commonUncommonWords = stats.commonUncommonWords2(text, lang, lowercase, contract, punct, stopWords)
		wordsClasses = stats.wordClass(text, lang, lowercase, contract, punct, stopWords)

		statistics = {'sentencesCount': stats.sentencesCount(text, lang),
							'charPerSent': stats.charactersPerSentence(text, lang, punct, contract, blank),
							'charPerWord': stats.charactersPerWord(text, lang, contract, punct),
							'wordsPerSent': stats.wordsPerSentence(text, lang, contract),
							'diffWords': diffWords[0],
							'conjunctions': wordsClasses[0],
							'adjectives': wordsClasses[1],
							'verbs': wordsClasses[2],
							'nouns': wordsClasses[3],
							'adverbs': wordsClasses[4],
							'prepositions': wordsClasses[5],
							'determiners': wordsClasses[6],
							'pronouns': wordsClasses[7],
							'interjections': wordsClasses[8],
							'shortWords': stats.shortWords(text, lang, contract, stopWords),
							#'commonWords': commonUncommonWords[0],
							#'uncommonWords': commonUncommonWords[1],
							}

		return statistics

	def __init__(self, text, lang = 'en', lowercase = True, punct = True, contract = True, stopWords = False, blank = False,
	 window = 500, limit = 0.72, segment = 100, coherence = False, extractor = None,):
		if coherence:
			self._cIndex = coh.coherenceMeasure(text, lang, extractor, punct, contract, stopWords)
		else:
			self._cIndex = 0
		self._fIndex = form.wfFormIndex(text, lang, lowercase, contract, stopWords)
		self._adjScore = form.adjScore(text, lang, lowercase, contract, stopWords)
		self._vIndex = {'TTR': vr.TTR(text, lang, lowercase, punct, contract, stopWords),
		 				'MSTTR': vr.MSTTR(text, lang, segment, lowercase, punct, contract, stopWords), 
		 				'MATTR': vr.MATTR(text, window, lang, lowercase, punct, contract, stopWords),
						'MTLD': vr.MTLD(text, limit, lang, lowercase, punct, contract, stopWords),
						'HDD': vr.HDD(text, lang, lowercase, punct, contract, stopWords)
						}
		self._rIndex = self._setReadabilityIndex(text, lang, contract)
		self._textStats = self._setTextStatistics(text, lang, lowercase, punct, contract, stopWords, blank)
		
		
		


	def getCoherenceIndex(self):
		return self._cIndex

	def getFormalityIndex(self):
		return self._fIndex

	def getAdjScore(self):
		return self._adjScore

	def getReadabilityIndex(self, index = ''):

		result = ''

		if index in self._rIndex:
			result = self._rIndex[index]
		elif index == '':
			result = self._rIndex
		else:
			result = 'ERROR: Bad Index'

		return result

	def getVocabularyIndex(self, index = ''):

		result = ''

		if index in self._vIndex:
			result = self._vIndex[index]
		elif index == '':
			result = self._vIndex
		else:
			result = 'ERROR: Bad Index'

		return result

	def getTextStatistics(self, index = ''):

		result = ''

		if index in self._textStats:
			result = self._textStats[index]
		elif index == '':
			result = self._textStats
		else:
			result = 'ERROR: Bad Index'

		return result



import contractions
import nltk
nltk.download('punkt')
nltk.download("popular")
import re
from nltk.tokenize import TweetTokenizer
from nltk.corpus import stopwords

def styleTokenizer(text, lang='es', token='word', lowercase = True, punct = True, contract = True, stopWords = True):
    """Splits in words a text.
   
    This function removes undesirable characters and extends English contractions.
    Then it appends each word to a list.
    Returns that list of words.
    Options:
        token: Default = 'word'
            If 'word', tokenize the text in words.
            If 'sent' or 'sentence', tokenize the text in sentences.
        lowercase: Defalut = True
            If True, the words are lower.
            If False, the words are not lower.
        punct (Only required if token='word'): Default = True
            If True: remove punctuation signs
            If False: no remove them
        contract(Only important if the text is in English): Default = True
            If True: expand english contractions
            If False: Does not expand them
        stopWords: Defalut = False
            If False: don't remove stopwords
            If True: remove stopwords   
    """
    #Lemmatizacion
    
    
    if lowercase:
        text = text.lower()
        
    text = re.sub(r'[/\\|#·@$~%&¬]', r'', text)
    text = re.sub(r'(?<=[.,?!;\–¿¡])(?=[^\s])', r' ', text)
    text = re.sub(r'[\W_]+ ,.\-\)\(><!?¡¿:;\]\[', '', text)
    text = re.sub(r'([\-–])', r'', text)
    text = re.sub(r'uf[0-9][0-9][0-9a-z]', r'', text)
    if contract and lang == 'en':
        text = contractions.fix(text)
        text = re.sub(r' \' ', r' ', text)
         
    text = re.sub('\s+', ' ', text).strip()
    text = re.sub(r' \'', r' ', text)

    if token == 'word':
        
        tokens = wTokenizer(text, lang, lowercase, punct, contract, stopWords)

    elif token == 'sent' or token == 'sentence':
        
        tokens = []

        if lang == "en":
            sent_detector = nltk.data.load('tokenizers/punkt/english.pickle')
       
        elif lang == "es":
            sent_detector = nltk.data.load('tokenizers/punkt/spanish.pickle')
        
        senTokens = sent_detector.tokenize(text.strip())
        senTokens = [x for x in senTokens if x]
        
        for sent in senTokens:
            aTokens = wTokenizer(sent, lang, lowercase, punct, contract, stopWords)
            tokens.append(' '.join(aTokens))
    
    return tokens

def removeStopWords(tokens, lang):
    """
    Removes the stop words in English or Spanish languages
    
    """
    if lang == "en":
        stopWords = set(stopwords.words("english"))
    elif lang == "es":
        stopWords = set(stopwords.words("spanish"))
    wordsFiltered = []

    for token in tokens:
        if token.lower() not in stopWords:
            wordsFiltered.append(token)
 
    return wordsFiltered

def wTokenizer(text, lang, lowercase, punct, contract, stopWords):
    
    if (contract and lang == 'en') or lang == 'es':
        tokens = nltk.word_tokenize(text)
    else:
        tknzr = TweetTokenizer()
        tokens = tknzr.tokenize(text)
    if punct:
        tokens = ([token for token in tokens if any(c.isalpha() or c is '+' for c in token)])
    if stopWords:
        tokens = removeStopWords(tokens, lang)
    
    return tokens


import nltk
from nltk.tag import StanfordPOSTagger
from nltk import FreqDist
#import wordfreq
from .styleTokenizer import styleTokenizer

def wordsCount(text = '', lang = 'en', contract = True):
    """
    Calculates the number of words in the text.
    
    """

    wordsNo = len(styleTokenizer(text, lang, contract = contract))
    
    return wordsNo

def sentencesCount(text = '', lang = 'en'):
    """
    Calculates the number of sentences in the text.

    """
    
    sentsNo = len(styleTokenizer(text, lang, token='sent'))

    return sentsNo


def charactersPerSentence(text = '', lang = 'en', punct = True, contract = False, blank = False):
    """
    Calculates an average of the number of characters per sentence.
    if blank: blankspace are counted as a character
    
    """
    charPerSent = 0
    sentsNo = sentencesCount(text,lang)
    sents = styleTokenizer(text, lang, token='sent', punct = punct, contract = contract)
    if punct:
        sentsAux = []
        puncts=[',', '.', ':', ';', '?', '¿', '¡','!', '-', '\"', '\'']
        for sent in sents:
            for c in puncts :
                if c in sent :
                    sent = sent.replace(c, '')
            sentsAux.append(sent)
        sents = sentsAux
    if blank:
        b = " "
    else: 
        b = ""
    for sent in sents:
        charPerSent += len(sent.replace(" ", b))
        
    charPerSent /= sentsNo
    
    return charPerSent

def charactersPerWord(text = '', lang = 'en', contract = False, punct = True):
    """Calculates an average of the number of characters per word.
    
    The function returns an average of the number of characters per word.
    
    """
    charPerWord = 0
    tokens = styleTokenizer(text, lang, contract = contract, punct = punct)
    
    wordsNo = len(tokens)
    
    for token in tokens:
        charPerWord += len(token)
        
    charPerWord /= wordsNo
    
    return charPerWord


def differentWords(text, lang, lowercase = True, contract = True, punct = True, stopWords = False):
    """Identifies the different words within the text.
    
    Returns the number of different words and a list with them.
    The function replaces uppercase with lowercase.
    
    """
    
    diffWordsList = []
    tokens = styleTokenizer(text, lang, lowercase = lowercase, contract = contract, punct = punct, stopWords = stopWords)
    tokens.sort()

    i = 0
    for token in tokens:
        if i+1 < len(tokens):
            if token != tokens[i + 1]:
                diffWordsList.append(token)
        else:
            diffWordsList.append(token)
        i += 1
    
    return len(diffWordsList), diffWordsList

def wordClass(text, lang, lowercase = True, contract = True, punct = True, stopWords = False):
    """It searchs the different types of words in the text
    
    It tokenizes the text in tagged words. Then, and thanks to the tags,
    it classifies the words by class and save the words in different lists.
    It returns the number of conjunctions, adjectives, verbs, nouns, adverbs, prepositions,
    determiners, pronouns and interjections..
    
    """

    tokens = styleTokenizer(text, lang, lowercase = lowercase, contract = contract, punct = punct, stopWords = stopWords)
    nouns = []
    verbs = []
    adjectives = []
    adverbs = []
    conjunctions = []
    prepositions = []
    determiners = []
    pronouns = []
    interjections =[]
    
    if lang == "en":
        
        taggedTokens = nltk.pos_tag(tokens)
        
        for tupla in taggedTokens:
            if tupla[1] == "CC":
                conjunctions.append(tupla[0])
            elif tupla[1][0] == "J":
                adjectives.append(tupla[0])
            elif tupla[1][0] == "V":
                verbs.append(tupla[0])
            elif tupla[1][0] == "N":
                nouns.append(tupla[0])
            elif tupla[1][0] == "R" or tupla[1] == "WRB":
                adverbs.append(tupla[0])
            elif tupla[1] == "TO" or tupla[1] == "IN":
                prepositions.append(tupla[0])
            elif tupla[1] == "WDT" or tupla[1] == "PDT" or tupla[1] == "DT":
                determiners.append(tupla[0])
            elif tupla[1] == "PRP" or tupla[1] == "PRP$" or tupla[1] == "WP" or tupla[1] == "WP$":
                pronouns.append(tupla[0])
            elif tupla[1] == "UH":
                interjections.append(tupla[0])
            
        result = [len(conjunctions), len(adjectives), len(verbs), len(nouns), len(adverbs), len(prepositions),
                  len(determiners), len(pronouns), len(interjections)]
            
            
    elif lang == "es":
        
        tagger = r"./stanfordPosTagger/models/spanish.tagger"
        jar = r"./stanfordPosTagger/stanford-postagger.jar"
        taggedTokens = StanfordPOSTagger(tagger,jar).tag(tokens)

        for tupla in taggedTokens:
            if tupla[1][0] == "c":
                conjunctions.append(tupla[0])
            elif tupla[1][0] == "a":
                adjectives.append(tupla[0])
            elif tupla[1][0] == "v":
                verbs.append(tupla[0])
            elif tupla[1][0] == "n":
                nouns.append(tupla[0])
            elif tupla[1][0] == "r":
                adverbs.append(tupla[0])
            elif tupla[1][0] == "s":
                prepositions.append(tupla[0])
            elif tupla[1][0] == "d":
                determiners.append(tupla[0])
            elif tupla[1][0] == "p":
                pronouns.append(tupla[0])
            elif tupla[1][0] == "i":
                interjections.append(tupla[0])
        
        result = [len(conjunctions), len(adjectives), len(verbs), len(nouns), len(adverbs), len(prepositions),
                  len(determiners), len(pronouns), len(interjections)]
    
    
    return result


def wordsPerSentence(text, lang, contract = False):
    """Calculates an average of the words per sentence.
    
    Return the average of words per sentence.
    
    """    
    sents = sentencesCount(text, lang)
    tokens = wordsCount(text, lang, contract = contract)

    wordsPerSent = tokens/sents
    
    return wordsPerSent

def shortWords(text, lang, contract = True, stopWords = False):
    """It counts the number of short words within the text.
    
    It returns the number of words with 3 characters or less (Short Words).
    
    """
    tokens = styleTokenizer(text, lang, contract = contract, stopWords = stopWords)

    shortWords = 0
    for token in tokens:
        if len(token) <= 3:
            shortWords += 1
            
    return shortWords

def commonUncommonWords(text, lang, lowercase = True, contract = True, punct = True, stopWords = False):
    """ Analyzes a text searching for common and uncommon words.
    
    It uses wordfreq library for searching common and uncommon words
    in English or Spanish languages.
    
    Returns the number of common and uncommon words.
    
    """
    diffWords = differentWords(text, lang, lowercase = lowercase, contract = contract, punct = punct, stopWords = stopWords)[1]
    commonWords = 0
    uncommonWords = 0
    
    for token in diffWords:
        if wordfreq.zipf_frequency(token, lang) > 2:
            commonWords += 1
        else:
            uncommonWords += 1
            
    return commonWords, uncommonWords

def freqWords(text, lang, lowercase = True, punct = True, contract = True, stopWords = False):
    """Calculates the frequency of each word within a text
    
    Returns de most common words.
    
    """
    tokens = styleTokenizer(text, lang, lowercase = lowercase, punct = punct, contract = contract, stopWords = stopWords)

    fdist = FreqDist(tokens)

    return fdist.most_common()


from . import TextStatistics as stats
import math
from scipy.stats import hypergeom
from .styleTokenizer import styleTokenizer


def TTR(text, lang, lowercase = True, punct = True, contract = True, stopWords = False):

    tokens = styleTokenizer(text, lang, lowercase = lowercase, punct = punct, contract = contract, stopWords = stopWords)
    wordsNumber = len(tokens)
    diffWords = stats.differentWords(text, lang, lowercase = lowercase, contract = contract, punct = punct, stopWords = stopWords)[0]
    TTR = (diffWords/wordsNumber)
    return TTR*100

def MSTTR(text, lang, segment, lowercase = True, punct = True, contract = True, stopWords = False):
    
    tokens = styleTokenizer(text, lang, lowercase = lowercase, punct = punct, contract = contract, stopWords = stopWords)
    wordsNumber = len(tokens)
    MSTTR = 0
    i = 0
    auxList = []
    segmentsNumber = 0
    if len(tokens)/segment > 1:
        for token in tokens:
            if len(auxList) < segment:
                auxList.append(token)              
                i += 1
            else:
                MSTTR += TTR(' '.join(auxList), lang)
                auxList = []
                segmentsNumber += 1
                
                if wordsNumber - i >= segment:
                    continue
                else:
                    break
        auxList = []
        j = 0
        while len(auxList) < wordsNumber - i:
            auxList.append(tokens[::-1][j])
            j += 1
        MSTTR += TTR(' '.join(auxList), lang)
        segmentsNumber += 1
        result = MSTTR/segmentsNumber
    else:
        result = 0
                
    return result

def MATTR(text, window, lang, lowercase = True, punct = True, contract = True, stopWords = False):
    tokens = styleTokenizer(text, lang, lowercase = lowercase, punct = punct, contract = contract, stopWords = stopWords)
    wordsNumber = len(tokens)
    if window > wordsNumber:
        result = 0
    else:
        MATTR = 0
        auxList = []
        j = 0
        while j + window <= wordsNumber:
            for i in range(j, window + j):
                auxList.append(tokens[i])
            j += 1
            MATTR += TTR(' '.join(auxList), lang)
            auxList = []
        result = MATTR/(j)

    return result

def MTLD(text, limit, lang, lowercase = True, punct = True, contract = True, stopWords = False):
    tokens = styleTokenizer(text, lang, lowercase = lowercase, punct = punct, contract = contract, stopWords = stopWords)
    wordsNumber = len(tokens)
    def MTLDimplementation(usedTokens, limit):
        nSegm = 0
        segment = []

        for i in range(0, wordsNumber):
            segment.append(usedTokens[i])
            if TTR(' '.join(segment), lang)/100 < limit:
                nSegm += 1
                segment = []
            elif i == wordsNumber - 1 and TTR(' '.join(segment), lang)/100 > limit:
                rest = 1 - TTR(' '.join(segment), lang)/100
                nSegm += rest/(1 - limit)
        return wordsNumber/nSegm
    
    MTLD1 = MTLDimplementation(tokens, limit)
    MTLD2 = MTLDimplementation(tokens[::-1], limit)
    
    result = (MTLD1 + MTLD2)/2

    if result > 100:
        return 100
    else:
        return result

def HDD(text, lang, lowercase = True, punct = True, contract = True, stopWords = False):
    tokens = styleTokenizer(text, lang, lowercase = lowercase, punct = punct, contract = contract, stopWords = stopWords)
    wordsNumber = len(tokens)
    freqWords = stats.freqWords(text, lang, lowercase = lowercase, punct = punct, contract = contract, stopWords = stopWords)
    probSum = 0
    k = 1
    lista = []
    for i in range(0, len(freqWords)):
        [N, d, n] = [wordsNumber, freqWords[i][1], 42]
        probType = hypergeom.pmf(k, N, d, n)
        probSum += probType
        lista.append(probType)  
        
    return probSum/42 *100